function LibLoginController(LibLoginService, LibUtilities, localStorageEngine, templateManager, eventAggregator,
                            ScreenService, metaDataFactory, WidgetsMetaData, ZhuAclController, ZhKeyboardService, ZHPromise, MessageService) {
    var self = this;
    var loginUserId, loginUserName, loginIsSupOper, loginSupOperId;
    window.tekLoginProjectId = angular.copy(window.projectId);

    var factoryExpiredSessionWidget = function() {
        return metaDataFactory.widgetFactory(WidgetsMetaData.getExpiredSession(), templateManager.container);
    };

    var factoryPHPErrorLogFilesWidget = function() {
        return metaDataFactory.widgetFactory(WidgetsMetaData.getPHPErrorLogFiles(), templateManager.container);
    };

    var updateLanguage = function () {
        var langParam = Util.getUrlParameter('lang');
        var language = langParam || localStorage.getItem('LOGIN_LANGUAGE') || window.lang;
        window.lang = language;
        ScreenService.setLanguage(language);
        localStorageEngine.setLocalVar('language', language);
    };

    var updateToken = function () {
        var authToken = Util.getUrlParameter('authToken');
        if (authToken) {
            localStorage.setItem('LOGIN_TOKEN', authToken);
        }
        return authToken;
    };

    var updateHashAndKeepConn = function () {
        var hash = Util.getUrlParameter('hash');
        if (hash) {
            localStorage.setItem('LOGIN_HASH', hash);
        }
        var keepConn = Util.getUrlParameter('keepConn');
        if (keepConn) {
            localStorage.setItem('LOGIN_KEEP_CONNECTED', keepConn);
        }
    };

    var updateSeparator = function() {
        var separator = Util.getUrlParameter('sep') || localStorage.getItem('LOGIN_SEPARATOR');
        if(separator) {
            localStorage.setItem('LOGIN_SEPARATOR', separator);
            if($.currency.i18n[window.lang]) {
                $.currency.i18n[window.lang].thousands = separator[0];
                $.currency.i18n[window.lang].inputThousands = separator[0];
                $.currency.i18n[window.lang].decimal = separator[1];
                $.currency.i18n[window.lang].inputDecimal = separator[1];
            }
        }
    };

    this.authenticate = function(forceAuthenticate) {
        templateManager.project.elementListDefaultRule = templateManager.project.elementListDefaultRule || 'I';
        updateLanguage();
        updateSeparator();
        updateHashAndKeepConn();

        var defer = ZHPromise.defer();
        forceAuthenticate = forceAuthenticate === true;

        if(window.location.hash.replace('#/', '') != LibUtilities.authenticationContainer || forceAuthenticate) {
            var setAclEventId;
            var user = localStorage.getItem('LOGIN_USER');
            var useRedirect = localStorage.getItem('LOGIN_USE_REDIRECT');
            var authToken = updateToken();

            if((!user && !authToken) || useRedirect == 'S') {
                LibUtilities.redirectToLogin();
                defer.resolve({});
            } else {
                var setAclFunction = function () {
                    setAcl(defer.promise, setAclEventId);
                };
                setAclEventId = eventAggregator.subscribe('onContainerOpen', setAclFunction);

                ScreenService.showLoader();
                LibLoginService.startSession(user).then(function (response) {
                    ScreenService.hideLoader();
                    startSessionResponse(response, defer);
                }).catch(function (error) {
                    ScreenService.hideLoader();
                    //Expired or Invalid Token
                    if(error.data && (error.data.errorCode == 101 || error.data.errorCode == 100)) {
                        self.logout();
                    } else if(error.data.errorCode == 20){
                        localStorage.setItem('LOGIN_ERASE_HASH', 'Y');
                        MessageService.showCustomDialogByCode('lib#ALE_SESSION_WILL_BE_ENDED', null, {
                            type: 'INFORMATION',
                            actions: [ {
                                label: ScreenService.i18n("Ok"),
                                code: function(){
                                    self.logout()
                                }
                            } ]
                        });
                    } else{
                        defer.reject(error);
                        LibUtilities.showErrorMessage(error);
                    }
                });
            }
        }

        return defer.promise;
    };

    var startSessionResponse = function(response, defer) {
        if (response.isError) {
            defer.reject(response.message);
        } else {
            if (!response.USER_ID) {
                self.logout();
            } else if (response.SESSION_EXPIRED) {
                logoutExpiredSession(true);
            } else {
                if(response.LICENSE_INFO) {
                    localStorage.setItem('LICENSE_INFO', response.LICENSE_INFO);
                }
                if(response.LOGIN_USER && response.LOGIN_TOKEN) {
                    localStorage.setItem('LOGIN_USER', response.LOGIN_USER);
                    localStorage.setItem('LOGIN_TOKEN', response.LOGIN_TOKEN);
                }
                
                localStorage.setItem('LOGIN_USER_NEXT', response.LOGIN_USER_NEXT || response.LOGIN_USER);
                localStorage.setItem('IS_SUPPORT_OPERATOR', response.IS_SUPPORT_OPERATOR);
                localStorage.setItem('LOGIN_USER_ID', response.USER_ID);
                localStorage.setItem('LOGIN_PHP_VERSION_ID', response.PHP_VERSION_ID);
                localStorage.setItem('ACL_ADMINISTRATOR', response.ACL_ADMINISTRATOR);
                localStorage.setItem('ACCESS_PROFILE', response.ACCESS_PROFILE);
                localStorage.setItem('LOGIN_CUSTOMIZATION_URL', response.CUSTOMIZATION_URL);
                localStorage.setItem('NOTICES', JSON.stringify(response.NOTICES));
                localStorage.setItem('TIMEOUTNOTICES', JSON.stringify(response.TIMEOUTNOTICES));

                if(response.IS_SUPPORT_OPERATOR) {
                    loginIsSupOper = response.IS_SUPPORT_OPERATOR;
                    loginSupOperId = response.SUPPORT_OPERATOR_ID;
                }
                if(response.IS_SUPPORT_OPERATOR || response.IS_TEKNISA_USER) {                    
                    LibUtilities.addPHPErrorLogButton();
                    ZhKeyboardService.bind('ctrl+alt+shift+l', self.getPHPErrorLogFiles);
                }

                loginUserId   = response.USER_ID;
                loginUserName = response.USER_NAME;

                if(response.IS_PENDENT_MIGRATION) {
                    MessageService.showMessageByCode('lib#ALE_MIGRATION_IS_PENDENT', null, { type: 'INFORMATION' });
                }
                
                if(response.VISIBLE_ELEMENTS) {
                    localStorage.setItem('LOGIN_VISIBLE_ELEMENTS', JSON.stringify(response.VISIBLE_ELEMENTS));
                }

                if(response.ACL_ADMINISTRATOR) {                    
                    LibUtilities.addTranslateButton();
                }

                defer.resolve(response);
            }
        }
    };

    this.pageViewNoticeActionEventCancel = function () {
        ScreenService.closePopup(true);
    };

    this.saveUserSilencers = function (args) {
        var row =  args.row
        if (Object.keys(row).length > 0) {
            restEngine.post("lib#/lib_saveUserNoticeSilence", {
                "requestType": "Row",
                "row": row
            }).then(function () {
                MessageService.showMessageByCode('lib#SUC_SAVED');
            }).catch(function(error) {
                ScreenService.hideLoader();
                LibUtilities.showErrorMessage(error);
            });
        }
    }

    this.readAsMark = function (args) {
        var nrinfecommerce = args.owner.widget.parent.currentWidget.name;
        var row = {
            NRINFECOMMERCE: nrinfecommerce,
            CDOPERADOR: localStorage.getItem('LOGIN_USER_ID')
        }
        if (markIsRead.includes(nrinfecommerce)) {
            MessageService.showMessageByCode('lib#ERR_MARK_IS_READ_NOTICE', [], {type: 'ERROR'});
        } else {
            restEngine.post("lib#/lib_readNotification", {
                "requestType": "Row",
                "row": row
            }).then(function () {
                markIsRead.push(nrinfecommerce);
                self.nextButton(args);
                MessageService.showMessageByCode('lib#SUC_MARK_IS_READ_NOTICE');
            }).catch(function(error) {
                ScreenService.hideLoader();
                LibUtilities.showErrorMessage(error);
            });
        }
    }

    this.previousButton = function(args) {
        args.owner.widget.parent.previousWidget();
        this.createLabelTeknisaStyle(args.owner.widget.parent.currentWidget);
    }

    this.nextButton = function(args) {
        args.owner.widget.parent.nextWidget();
        this.createLabelTeknisaStyle(args.owner.widget.parent.currentWidget);
    }

    this.mountedSlidesNotices = function (
        pageViewNoticeWidgetObj, noticesArray, noticesTeknisa, classTeknisa, visibilityTeknisa,
        noticesInternal, classInternal, visibilityInternal, visibilitySave
    ) {
        noticesArray.forEach(function(notices) {
            var isTeknisa = "N";
            if (notices.CDPUBLICOALVO =='T') {
                isTeknisa = "S";
            }
            var slide = {
                name: notices.NRINFECOMMERCE,
                label: notices.DSTITULOINF,
                template: "widget/form.html",
                isVisible: true,
                isTeknisa: isTeknisa,
                id: notices.NRINFECOMMERCE,
                showDescriptionOrder: false,
                actions: [
                    {
                        name: "prev_" + notices.NRINFECOMMERCE,
                        label: ScreenService.i18n("Anterior"),
                        icon: "previous",
                        showAsAction: "always|view",
                        events: [{ name: "ActionEvent", code: "LibLoginController.previousButton(args)" }]
                    },
                    {
                        name: "next_" + notices.NRINFECOMMERCE,
                        label: ScreenService.i18n("Próximo"),
                        icon: "forward",
                        showAsAction: "always|view",
                        events: [{ name: "ActionEvent", code: "LibLoginController.nextButton(args)" }]
                    },
                    {
                        name: "close_" + notices.NRINFECOMMERCE,
                        label: ScreenService.i18n("Fechar"),
                        isVisible: true,
                        hideIcon: true,
                        showAsAction: "back",
                        events: [{ name: "ActionEvent", code: "LibLoginController.pageViewNoticeActionEventCancel()" }]
                    },
                    {
                        name: "markisread_" + notices.NRINFECOMMERCE,
                        label: ScreenService.i18n("Marcar como Lido"),
                        isVisible: true,
                        showAsAction: "never",
                        events: [{ name: "ActionEvent", code: "LibLoginController.readAsMark(args)" }]
                    },
                ],
                events: [],
                fields: [
                    {
                        name: "noticesForTeknisaViewNotice",
                        id: "selectTeknisaViewNotice",
                        label: ScreenService.i18n("Silenciar comunicados Teknisa"),
                        template: "field/select-native.html",
                        valueField: "ID",
                        descriptionField: "DSNOME",
                        dataSource: {
                            data: noticesTeknisa,
                        },
                        class: classTeknisa,
                        readOnly: false,
                        isVisible: visibilityTeknisa,
                        showOnList: true,
                        showOnForm: true,
                    },
                    {
                        name: "noticesForInternalViewNotice",
                        id: "selectInternalViewNotice",
                        label: ScreenService.i18n("Silenciar comunicados Internos"),
                        template: "field/select-native.html",
                        valueField: "ID",
                        descriptionField: "DSNOME",
                        dataSource: {
                            data: noticesInternal,
                        },
                        class: classInternal,
                        readOnly: false,
                        isVisible: visibilityInternal,
                        showOnList: true,
                        showOnForm: true,
                    },
                    {
                        name: "buttonSaveSilencer",
                        label: ScreenService.i18n("Salvar"),
                        template: "field/button.html",
                        class: 2,
                        readOnly: false,
                        isVisible: visibilitySave,
                        showOnList: true,
                        showOnForm: true,
                        events: [
                            {
                                name: "ActionEvent", 
                                code: "LibLoginController.saveUserSilencers(args)" 

                            }
                        ]
                    }
                ]
            };
    
            switch (notices.IDNOTIFICACAO) {
                case "H":
                    slide.fields.push({
                        name: "text_" + notices.NRINFECOMMERCE,
                        label: "",
                        class: 12,
                        template: "field/text-raw.html",
                        text: notices.TXCONTEUDOINF,
                        isVisible: true
                    });
                    break;
                case "I":
                    var imageField = {
                        name: "image_" + notices.NRINFECOMMERCE,
                        id: "image_" + notices.NRINFECOMMERCE,
                        label: "",
                        isVisible: true,
                        height: "625px",
                        width: "100%",
                        objectFit: "fill",
                        source: notices.TXCONTEUDOINF,
                        template: "field/image-static.html"
                    };
                    
                    if (notices.LINKDIRIMAGE) {
                        imageField.link = {
                            to: notices.LINKDIRIMAGE,
                            target: "_blank"
                        };
                    }
                    
                    slide.fields.push(imageField);
                    break;
                case "P":
                    slide.fields.push({
                        name: "pdf_" + notices.NRINFECOMMERCE,
                        label: "",
                        class: 12,
                        template: "field/textarea.html",
                        readOnly: true,
                        isVisible: true,
                        showOnList: false,
                        showOnView: true,
                        showOnForm: true,
                        toolbar: {
                            undo: false,
                            redo: false,
                            bold: false,
                            italic: false,
                            underline: false,
                            superscript: false,
                            subscript: false,
                            strikethrough: false,
                            clear: false,
                            fontname: false,
                            fontsize: false,
                            color: false,
                            ul: false,
                            ol: false,
                            paragraph: false,
                            lineheight: false,
                            table: false,
                            picture: false,
                            codeview: false
                        }
                    });
                    break;
                case "V":
                    slide.fields.push({
                        name: "video_" + notices.NRINFECOMMERCE,
                        label: "",
                        class: 12,
                        template: "field/textarea.html",
                        readOnly: true,
                        isVisible: true,
                        showOnList: false,
                        showOnView: true,
                        showOnForm: true,
                        toolbar: {
                            undo: false,
                            redo: false,
                            bold: false,
                            italic: false,
                            underline: false,
                            superscript: false,
                            subscript: false,
                            strikethrough: false,
                            clear: false,
                            fontname: false,
                            fontsize: false,
                            color: false,
                            ul: false,
                            ol: false,
                            paragraph: false,
                            lineheight: false,
                            table: false,
                            picture: false,
                            codeview: false
                        }
                    });
                break;
                default:
                    console.warn("Tipo de notificação desconhecido");
                    break;
            }
            pageViewNoticeWidgetObj.widgets.push(slide);
        });
        var eventOnEnter = {
            name: "WidgetOnEnter",
            code: "LibLoginController.createLabelTeknisaStyle(args.owner)"
        }
        pageViewNoticeWidgetObj.widgets[0].events.push(eventOnEnter);
        return pageViewNoticeWidgetObj;
    }

    var createdStyleSlidesNotices = function (noticesArray, visibilitySave) {
        noticesArray.forEach(function(notices) {
            var element = $(".zh-" + notices.NRINFECOMMERCE).parent();
            element[0].style.height = "100%";
            element[0].style.maxHeight = "100%";
            element[0].style.paddingRight = 0;
            var silencer = $("form[data-zh-id="+notices.NRINFECOMMERCE+"] .row");
            var silencerHeight =  0;
            if (visibilitySave == true) {
                silencer[0].style.marginTop = "10px";
                var silencerHeight = $("form[data-zh-id="+notices.NRINFECOMMERCE+"] .row").height();
            }
            var bodyHeight = element.height() -  $('.zh-footer-center').parent().height();
            var heightImage = bodyHeight - silencerHeight - 29;
            var heightPDF = bodyHeight - silencerHeight - 32;
            var heightNoticesImg = heightImage + 27;
            var heightNoticesPdf = heightPDF + 37;
            var heightCalcImg = String(heightNoticesImg + "px");
            var heightCalcPdf = String(heightNoticesPdf + "px");
            var url = "";
            var nameFieldElement = "";
            if (notices.IDNOTIFICACAO === 'P') {
                url = notices.TXCONTEUDOINF;
                nameFieldElement = "pdf_" + notices.NRINFECOMMERCE
            }
            if (notices.IDNOTIFICACAO === 'V') {
                url = convertToEmbedLink(notices.TXCONTEUDOINF);
                nameFieldElement = "video_" + notices.NRINFECOMMERCE
            }
            if (notices.IDNOTIFICACAO === 'I') {
                var img = $('div[data-zh-id="image_' + notices.NRINFECOMMERCE + '"] img')[0];
                img.style.height = heightCalcImg;
                img.style.width = "100%";
            }
            if (notices.IDNOTIFICACAO === 'H') {
                var text = $("#text_" + notices.NRINFECOMMERCE)[0];
                text.style.height = heightCalcImg;
                text.style.border = "solid 1px";
                text.style.padding = "10px";
            }
            if (url !== "" && nameFieldElement !== "") {
                var htmlPDF = "<iframe id="+ notices.NRINFECOMMERCE + "viewNotice src = "+ url + "#page=1&toolbar=0&navpanes=0' type='application/pdf' style= 'width: 100%; height: "+ heightCalcPdf +"; position: relative'></iframe>";
                var fieldSelector = $("#" + nameFieldElement);
                divField = fieldSelector.parent()[0];
                fieldSelector.parent()[0].innerHTML = htmlPDF;
            }
        });
    }

    this.openPopupNotices = function(noticesArray, timeOutnotices) {
        var pageViewNoticeWidgetObj = WidgetsMetaData.getPageViewNotice();
        var visibilitySave = false;
        var visibilityTeknisa = false;
        var visibilityInternal = false;
        var classTeknisa = 10;
        var classInternal = 10;
        var noticesInternal = [];
        var noticesTeknisa = [];
        if (timeOutnotices.length > 0) {
            timeOutnotices.forEach(function(element){
                if (element.CDPUBLICOALVO === 'LOGIN') {
                    noticesInternal.push(element);
                }
                if (element.CDPUBLICOALVO === 'OM') {
                    noticesTeknisa.push(element);
                }
            });
            if (noticesInternal.length > 0 && noticesTeknisa.length > 0) {
                visibilityTeknisa = true;
                visibilityInternal = true;
                visibilitySave = true;
                classInternal = 5;
                classTeknisa = 5;
            } else if (noticesInternal.length > 0) {
                visibilityInternal = true;
                classInternal = 10;
                visibilitySave = true;
                visibilityTeknisa = false;
            } else if (noticesTeknisa.length > 0) {
                visibilityTeknisa = true;
                classTeknisa = 10;
                visibilitySave = true;
                visibilityInternal = false;
            }
        }
        pageViewNoticeWidgetObj = this.mountedSlidesNotices (
            pageViewNoticeWidgetObj, noticesArray, noticesTeknisa, classTeknisa, visibilityTeknisa,
            noticesInternal, classInternal, visibilityInternal, visibilitySave
        );
        var pageViewNoticeWidget = metaDataFactory.widgetFactory(pageViewNoticeWidgetObj, templateManager.container);
        ScreenService.openPopup(pageViewNoticeWidget).then(function () {
            Util.doAsync(function () {
                createdStyleSlidesNotices(noticesArray, visibilitySave);
            }, 2000);
        });
    }

    var convertToEmbedLink = function(url) {
        var isYouTubeLink = url.includes("youtube.com") || url.includes("youtu.be");
        if (!isYouTubeLink) {
            return url;
        }
        var videoIdMatch = url.match(/(?:youtube\.com\/.*[?&]v=|youtu\.be\/)([\w-]{11})/);
        if (videoIdMatch && videoIdMatch[1]) {
            var videoId = videoIdMatch[1];
            return "https://www.youtube.com/embed/" + videoId;
        }
        return '';
    }

    this.createLabelTeknisaStyle = function(widget) {
        Util.doAsync(function () {
            var popupTitle = $('.popup-title')[0];
            if (widget.isTeknisa == 'S') {
                var popupHeader = $('.popup-container header')[0];
                popupHeader.style.height = "46px";
                // Criar o elemento <i>
                var labelTeknisa = document.createElement("i");
                labelTeknisa.id = "comunicado-italico";
                labelTeknisa.textContent = " " + ScreenService.i18n("Comunicado Teknisa"); // Adiciona o texto

                // Estilos da tag <i>
                labelTeknisa.style.fontStyle = "italic";
                labelTeknisa.style.display = "block";
                labelTeknisa.style.position = "relative";
                labelTeknisa.style.textAlign = "right";
                labelTeknisa.style.fontWeight = "bold";
                labelTeknisa.style.fontSize = "16px";
                labelTeknisa.style.bottom = "16.5px";
                labelTeknisa.style.opacity = "0.4";

                // Adiciona o elemento <i> ao final da div, sem apagar o conteúdo existente
                popupTitle.appendChild(labelTeknisa);
            }
        }, 500);
    }

    this.logout = function() {
        ScreenService.showLoader();
        LibLoginService.logout().then(function () {
            ScreenService.hideLoader();
            localStorage.removeItem('ACL_ADMINISTRATOR');
            localStorage.removeItem('ACCESS_PROFILE');
            localStorage.removeItem('IS_LOGGED');
            localStorage.removeItem('LOGIN_USER');
            localStorage.removeItem('LOGIN_USER_NEXT');
            localStorage.removeItem('LOGIN_TOKEN');
            localStorage.removeItem('LICENSE_INFO');
            localStorage.removeItem('LOGIN_ERASE_HASH');
            localStorage.removeItem('IS_SUPPORT_OPERATOR');
            localStorage.removeItem('LOGIN_USER_ID');
            localStorage.removeItem('LOGIN_PHP_VERSION_ID');
            localStorage.removeItem('LIB_FAVORITES_DATA');
            localStorage.removeItem('NOTICES');
            localStorage.removeItem('TIMEOUTNOTICES');
            LibUtilities.redirectToLogin();
        }).catch(function(error) {
            ScreenService.hideLoader();
            LibUtilities.showErrorMessage(error);
        });
    };
 
    var setAcl = function (loginPromise, setAclEventId) {
        loginPromise.then(function(response) {
            if(!window.productApplyAcl) {
                ZhuAclController.setUserRules(response.ACCESS_PROFILE, window.projectId, response.USER_ID, response.ORGANIZATION_ID,
                    {
                        canConfigureAcl: response.ACL_ADMINISTRATOR,
                        rules          : response.ACL_ADMINISTRATOR ? [] : false,
                        defaultRule    : response.ACL_ADMINISTRATOR ? 'A' : false
                    }
                );
                templateManager.project.aclDefaultRule = 'A';
            }
            eventAggregator.unSubscribe('onContainerOpen', setAclEventId);
        }).catch(LibUtilities.showErrorMessage);
    };

    eventAggregator.subscribe("LibExpiredSession", function () {
        if(!templateManager.container.getActiveWidget().tekLibExpiredSession) {
            var expiredSessionWidget = factoryExpiredSessionWidget();
            expiredSessionWidget.attempts = 0;
            expiredSessionWidget.dataSource.data = [];
            expiredSessionWidget.newRow();
            expiredSessionWidget.currentRow.OPERATOR = loginUserId + ' - ' + loginUserName;
            ScreenService.openPopup(expiredSessionWidget);
            localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', true);

        }
    });

    eventAggregator.subscribe("LibInvalidToken", function (errorCode) {
        //Expired Token
        if(errorCode == 101) {
            MessageService.showCustomDialogByCode('lib#ALE_EXPIRED_SESSION', null, {
                type: 'INFORMATION',
                actions: [ {
                    label: ScreenService.i18n("Ok"),
                    code: self.logout
                } ]
            });
        }
        //Invalid Token
        else if(errorCode == 100) {
            self.logout()
        }
    });

    eventAggregator.subscribe("LibInvalidTokenSupportOperator", function (errorCode) {
        //Expired Token Support Operator
        MessageService.showCustomDialogByCode('lib#ALE_EXPIRED_SESSION_SUPPORT_OPERATOR', null, {
            type: 'INFORMATION',
            actions: [{
                label: ScreenService.i18n("Ok"),
                code: self.logout
            }]
        });
    });

    eventAggregator.subscribe("LibConcurrentAccess", function () {
        localStorage.setItem('LOGIN_ERASE_HASH', 'Y');
        MessageService.showCustomDialogByCode('lib#ALE_SESSION_WILL_BE_ENDED', null, {
            type: 'INFORMATION',
            actions: [ {
                label: ScreenService.i18n("Ok"),
                code: function(){
                    self.logout()
                }
            } ]
        });
    });

    eventAggregator.subscribe("LibAccessBlocked", function (errorCode) {
        MessageService.showCustomDialogByCode('lib#ACCESS_BLOCKED_BY_TIME_OR_DAY', null, {
            type: 'INFORMATION',
            actions: [ {
                label: ScreenService.i18n("Ok"),
                code: function(){
                    self.logout()
                }
            } ]
        });
    });

    var logoutExpiredSession = function (afterLogin) {
        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
        var message = afterLogin ? 'ERR_EXPIRED_SESSION_LOGIN' : 'ERR_ATTEMPT_END';
        MessageService.getMessageByCode('lib#' + message).then(function(loadingMessage) {
            ScreenService.changeLoadingMessage(ScreenService.i18n(loadingMessage));
            ScreenService.showLoader();
            Util.doAsync(self.logout, 5000);
        });
    };

    this.expiredSessionActionEventConfirm = function(args) {
        var expiredSessionWidget = args.owner.widget ? args.owner.widget : args.owner.field.widget;
        if(expiredSessionWidget.isValid()) {
            var password = expiredSessionWidget.currentRow.PASSWORD;
            var userId   = loginSupOperId || loginUserId;
            ScreenService.showLoader();
            LibLoginService.validateUser(userId, password, loginIsSupOper)
                .then(function() {
                    // Requisição getModulesVersion chamada para atualizar o tempo de sessão local (produto ou módulo)
                    LibLoginService.getModulesVersion();
                    ScreenService.hideLoader();
                    localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                    ScreenService.closePopup();
                }).catch(function(error) {
                    ScreenService.hideLoader();
                    if(error.data && error.data.errorCode && error.data.errorCode == 20) {
                        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                        eventAggregator.publish('LibConcurrentAccess');
                    } else if(error.data && error.data.errorCode && error.data.errorCode == 102) {
                        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                        eventAggregator.publish('LibInvalidTokenSupportOperator');
                    } else if(error.data && error.data.errorCode && (error.data.errorCode == 21 || error.data.errorCode == 22)) {
                        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                        eventAggregator.publish('LibAccessBlocked');
                    } else {
                        var params = {attempts: ++expiredSessionWidget.attempts};
                        MessageService.showMessageByCode('lib#ERR_ATTEMPT_PARTIAL', params, {type: 'ERROR'});
                        if (expiredSessionWidget.attempts >= 3) {
                            logoutExpiredSession();
                        }
                    }
                });
        }
    };

    this.expiredSessionActionEventCancel = function() {
        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
        self.logout();
    };

    this.expiredSessionFieldOnKeyupPassword = function(args) {
        var key = window.event.keyCode;

        //13 = ENTER
        if(key == 13) {
            this.expiredSessionActionEventConfirm(args);
        }
    };

    this.downloadModulesVersionFile = function() {
        return LibLoginService.generateModulesVersionFile().then(function(response) {
            if(response && response.dataset) {
                var fileName = response.dataset.generatedFile[0];
                LibLoginService.downloadModulesVersionFile(fileName);
                return true;
            } else {
                LibUtilities.showErrorMessage(response);
                return false;
            }
        }).catch(LibUtilities.showErrorMessage);
    };

    /**
     * @param user -> Object or null.
     *                {
     *                    NAME:  'User Name',
     *                    EMAIL: 'user.email@teknisa.com'
     *                }
     * @param accessProfile -> Array of objects or null. ID is the column NRPERFILACESSO from table PERFILACESSO.
     *                         [
     *                             {
     *                                 ID: '1',
     *                                 IDPERFILACESSOADM: 'S'
     *                             },
     *                             {
     *                                 ID: '2',
     *                                 IDPERFILACESSOADM: 'N'
     *                             }
     *                         ]
     * @param otherSessionValues -> Object or null. PS.: CDFILIALSELECIONADA not exists in session. Use CDFILIAL.
     *                              {
     *                                  CDFILIAL:  '9999',
     *                                  NRPARCNEGOCIO: '1',
     *                                  CDPICTPROD: 'a.a.aa.aaa'
     *                              }
     * @returns Promise
     */
    this.updateSession = function(user, accessProfile, otherSessionValues) {
        var defer = ZHPromise.defer();
        ScreenService.showLoader();
        LibLoginService.updateSession(user, accessProfile, otherSessionValues).then(function(response) {
            ScreenService.hideLoader();
            if(response && response.dataset && response.dataset.sessionUpdated) {
                var sessionUpdated = response.dataset.sessionUpdated[0];
                localStorage.setItem('LOGIN_USER', sessionUpdated);
                defer.resolve();
            } else {
                defer.reject(response);
            }
        }).catch(function(error) {
            ScreenService.hideLoader();
            defer.reject(error);
        });

        return defer.promise;
    };

    this.expiredMenuOnClick = function() {
        MessageService.showMessageByCode('lib#ALE_EXPIRED_MENU', null, { type: 'ERROR' } );
    };

    this.updateTranslation = function() {
        MessageService.confirmMessageByCode('lib#ALE_UPDATE_TRANSLATION', null,
            {
                type: 'CONFIRMATION',
                success: function() {
                    ScreenService.showLoader();
                    LibLoginService.updateTranslation().then(function () {
                        var widget = templateManager.container.getActiveWidget();
                        if(self.checkContainerIsZhNext() && $('iframe').length && widget.sourceUrl) {
                            window.libTranslateDefer = ZHPromise.defer();
                            var frameUrl = widget.sourceUrl.split('?')[0];
                            $('iframe')[0].contentWindow.postMessage({ type: 'updateTranslation' }, frameUrl);
                            window.libTranslateDefer.then(function() {
                                reloadUpdateTranslation();
                            }).catch(function(error) {
                                ScreenService.hideLoader();
                                LibUtilities.showErrorMessage(error);
                            }); 
                        } else {
                            reloadUpdateTranslation();
                        }
                    }).catch(function(error) {
                        ScreenService.hideLoader();
                        LibUtilities.showErrorMessage(error);
                    });
                },
                fail: function () {}
            }
        );
    };

    var reloadUpdateTranslation = function() {
        MessageService.showMessageByCode('lib#SUC_UPDATE_TRANSLATION');
        setTimeout(function() {
            window.location.reload(true);
        }, 7000);
    };

    this.checkContainerIsZhNext = function() {
        if(templateManager.container.widgets && templateManager.container.widgets.length) {
            return templateManager.container.widgets[0].template == 'widget/iframe.html';
        }
        return false;
    };

    this.downloadPHPErrorLog = function(args) {
        var url = window.serviceUrl + '/lib_downloadPHPErrorLog'
                                    + '?token=' + localStorage.getItem('LOGIN_TOKEN')
                                    + '&filename=' + args.row.NAME;
        window.open(url);
    };

    this.getPHPErrorLogFiles = function() {
        ScreenService.showLoader();
        LibLoginService.getPHPErrorLogFiles().then(function (response) {
            var widget = factoryPHPErrorLogFilesWidget();
            widget.dataSource.data = response.dataset.getPHPErrorLogFiles;
            ScreenService.hideLoader();
            ScreenService.openPopup(widget);
            if(ScreenService.isSideMenuOpen()) {
                ScreenService.toggleSideMenu();
            }
        }).catch(function(error) {
            ScreenService.hideLoader();
            LibUtilities.showErrorMessage(error);
        });
    };

    this.phpErrorLogFilesActionEventCancel = function () {
        ScreenService.closePopup(true);
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibLoginController', LibLoginController);
});